/***************************************************************************
 *                              SEU RoboCup Simulation Team
 *                     -------------------------------------------------
 * Copyright (c) Southeast University , Nanjing, China.
 * All rights reserved.
 *
 * $Id: Step.h 1894 2008-07-02 13:09:05Z xy $
 *
 ****************************************************************************/

#ifndef TASK_STEP_H
#define TASK_STEP_H

#define ENABLE_TASK_STEP_LOG

#include "Task.h"
#include "math/Math.hpp"
#ifdef ENABLE_TASK_STEP_LOG
#include "logger/Logger.h"
#else
#include "logger/NoLogger.h"
#endif

namespace task {

    class Step: public Task
    {
    public:
        Step( bool isLeft,
              const seumath::Vector2f& size, seumath::AngDeg dir,
              boost::shared_ptr<const Step> preStep,
              float bodyHeight,
	      bool goStop,
              Task* primary
	      );

        virtual bool isTerminable() const;

        virtual bool isDone() const;

        virtual boost::shared_ptr<action::Action> perform();

        /**
         * @return moving which foot
         */
        bool isLeft() const
            {
                return mIsLeft;
            }

        /**
         * @return the size of the step
         */
        const seumath::Vector2f& preSize() const
            {
                return mPreSize;
            }

        const seumath::Vector2f& preSizeAcc() const
            {
                return mPreSizeAcc;
            }

        seumath::AngDeg predir() const
            {
                return mPreDir;
            }

        /**
         * @return the size of the step
         */
        const seumath::Vector2f& size() const
            {
                return mSize;
            }

        const seumath::Vector2f& sizeAcc() const
            {
                return mSizeAcc;
            }

        /**
         * @return the direction of the step
         */
        seumath::AngDeg dir() const
            {
                return mDir;
            }
            
        seumath::AngDeg dirAcc() const 
	    {
		return mDirAcc;
	    }
            
        int walkType() const
	    {
		return mWalkType;
	    }
        
        static void setMaxSizeAcc( const seumath::Vector2f& v )
            {
                mMaxSizeAcc = v;
            }

        static void setMaxSize( const seumath::Vector2f& v )
            {
                mMaxSize = v;
            }

        static void setMaxDirAcc(seumath::AngDeg a)
            {
                mMaxDirAcc = a;
            }

        static void setMaxDir(seumath::AngDeg a)
            {
                mMaxDir = a;
            }

        static const seumath::Vector2f& getMaxSizeAcc()
            {
                return mMaxSizeAcc;
            }

        static const seumath::Vector2f getMaxSize()
            {
                return mMaxSize;
            }

        static seumath::AngDeg getMaxDirAcc()
            {
                return mMaxDirAcc;
            }

        static seumath::AngDeg getMaxDir()
            {
                return mMaxDir;
            }

        static seumath::AngDeg getStepTime()
            {
                return mStepTime;
            }
           
//
    protected:

        /// configuration of the walking step
        static seumath::Vector2f mMaxSizeAcc;
        static seumath::Vector2f mMinSize;
        static seumath::Vector2f mMaxSize;
        static seumath::AngDeg mMaxDirAcc;
        static seumath::AngDeg mMaxDir;
        static seumath::AngDeg mMinDir;
        static float mStepTime;
	
	//ghd
	static seumath::Vector2f mMaxSize2Acc;
	static seumath::AngDeg mMaxDir2Acc;
	
	/// next step
	/// steping left foot or right foot
        bool mIsLeft;

        /// the step size
        seumath::Vector2f mSize;

        /// the size accerelation
        seumath::Vector2f mSizeAcc;

        /// the direction change
        seumath::AngDeg mDir;
	
	//ghd
	seumath::AngDeg mDirAcc;
	
	///current step
	seumath::Vector2f mPreSize;
        seumath::Vector2f mPreSizeAcc;
        seumath::AngDeg mPreDir;
	seumath::AngDeg mPreDirAcc;
	
	seumath::Vector2f mTempSize;
	
	///0 : front-side
	///1 : front
	///2 : side
	///3 : turn (don't walk)
	///4 : stop (when the match begin)
	int mWalkType;
	
	bool mHaveLast;	//is it have last step
	

    private:
        // this class handls logging
        DECLARE_STATIC_GRAPHIC_LOGGER;
    };

} // namespace task


#endif // TASK_STEP_H

